@@ -5,7 +5,7 @@
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>
 ;; Maintainer: Protesilaos Stavrou <info@protesilaos.com>
 ;; URL: https://github.com/protesilaos/beframe
-;; Version: 1.1.1
+;; Version: 1.2.1
 ;; Package-Requires: ((emacs "28.1"))
 
 ;; This file is NOT part of GNU Emacs.
@@ -127,38 +127,91 @@ automatically, use `customize-set-variable' or `setopt' (Emacs
          (beframe--functions-in-frames))
   :type 'symbol)
 
-(defun beframe--frame-buffers (&optional frame)
-  "Produce list of buffers for either specified or current FRAME."
-  (seq-filter
-   (lambda (buf)
-     (and (bufferp buf)
-          (not (string-prefix-p " " (buffer-name buf)))))
-   (frame-parameter frame 'buffer-list)))
+(defcustom beframe-kill-buffers-no-confirm nil
+  "When non-nil, do not prompt for confirmation when killing buffers.
+This concerns the command `beframe-kill-buffers-matching-regexp'.
 
-(defun beframe--public-buffers ()
-  "Return list of buffers from all frames.
-This is the same as the output of the `buffer-list' function
-minus all the internal buffers."
-  (seq-filter
-   (lambda (buf)
-     (and (bufferp buf)
-          (not (string-prefix-p " " (buffer-name buf)))))
-   (buffer-list)))
+If nil (the default), `beframe-kill-buffers-matching-regexp' asks for
+confirmation once and then proceeds to kill all the buffers it has found.
+
+Emacs may still prompt to confirm each action if the buffer is unsaved,
+has a running process, and the like."
+  :group 'beframe
+  :package-version '(beframe . "1.2.0")
+  :type 'boolean)
+
+(defun beframe--remove-internal-buffers (buffers)
+  "Remove internal buffers from BUFFERS."
+  (seq-remove
+   (lambda (buffer)
+     (string-prefix-p " " (buffer-name buffer)))
+   buffers))
+
+(defun beframe--get-buffers-matching-regexp (regexp &optional match-mode-names no-internal-buffers)
+  "Return buffers whose name matches REGEXP.
+With optional MATCH-MODE-NAMES also return buffers whose major mode
+matches REGEXP.
+
+With optional NO-INTERNAL-BUFFERS, get the `buffer-list' filtered
+through `beframe--remove-internal-buffers'."
+  (let ((buffers (if no-internal-buffers
+                     (beframe--remove-internal-buffers (buffer-list))
+                   (buffer-list))))
+    (seq-filter
+     (lambda (buffer)
+       (if match-mode-names
+           (or (string-match-p regexp (buffer-name buffer))
+               (with-current-buffer buffer
+                 (string-match-p regexp (symbol-name major-mode))))
+         (string-match-p regexp (buffer-name buffer))))
+     buffers)))
+
+(defun beframe--get-buffers (&optional arg)
+  "Return list of buffers from different sources depending on ARG.
+
+The following values of ARG can be used:
+
+- A list of one to three elements that are passed to the function
+  `beframe--get-buffers-matching-regexp' (refer to its doc string from
+  the arguments).
+
+- A string, which is passed to `beframe--get-buffers-matching-regexp' as
+  the sole argument.
+
+- The symbol \\='public\\=' to filter the `buffer-list' through
+  `beframe--remove-internal-buffers'.
+
+- The symbol \\='global\\=' to get the return value of the function
+  `beframe--global-buffers', which reads the user option
+  `beframe-global-buffers'.
+
+- nil or a frame object satisfying `frame-live-p' to get the
+  \\='buffer-list\\=' parameter of either the `selected-frame' or the
+  given frame object, filtered through `beframe--remove-internal-buffers'."
+  (pcase arg
+    ((or `(,regexp ,match-major-modes ,no-internal-buffers)
+         `(,regexp ,match-major-modes)
+         `(,regexp)
+         (and (pred stringp) `,regexp))
+     (beframe--get-buffers-matching-regexp regexp match-major-modes no-internal-buffers))
+    ('public (beframe--remove-internal-buffers (buffer-list)))
+    ('global (beframe--global-buffers))
+    ((or (pred null) (pred frame-live-p))
+     (beframe--remove-internal-buffers (frame-parameter arg 'buffer-list)))
+    (_ (user-error "Wrong argument in `beframe--get-buffers' pcase"))))
 
 (defun beframe--global-buffers ()
   "Return list of `beframe-global-buffers' buffer objects."
-  (mapcan
-   (lambda (regexp-or-symbol)
-     (seq-filter
-      (lambda (buffer)
-        (when (or (and (stringp regexp-or-symbol)
-                       (string-match-p regexp-or-symbol (buffer-name buffer)))
-                  (and (symbolp regexp-or-symbol)
-                       (with-current-buffer buffer
-                         (derived-mode-p regexp-or-symbol))))
-          buffer))
-      (beframe--public-buffers)))
-   beframe-global-buffers))
+  (pcase-let* ((pub-buffs (beframe--get-buffers 'public))
+               (`(,str-re ,modes-lst)
+                (cl-loop for E in beframe-global-buffers
+                         if (stringp E) collect E into str
+                         else if (symbolp E) collect E into sym
+                         finally return (list (string-join str "\\|") sym))))
+    (cl-loop for b in pub-buffs
+             if (string-match-p str-re (buffer-name b)) collect b
+             else if (with-current-buffer b (derived-mode-p modes-lst))
+             collect b)))
 
 (cl-defun beframe-buffer-list (&optional frame &key sort)
   "Return list of buffers that are used by the current frame.
@@ -174,7 +227,7 @@ Include `beframe-global-buffers' in the list."
   (funcall (or sort #'identity)
            (delq nil
                  (delete-dups
-                  (append (beframe--frame-buffers frame)
+                  (append (beframe--get-buffers frame)
                           (beframe--global-buffers))))))
 
 (define-obsolete-function-alias
@@ -191,7 +244,7 @@ more information."
 
 (defun beframe--buffer-names-consolidated ()
   "Return list of names of all buffers as strings."
-  (mapcar #'buffer-name (beframe--public-buffers)))
+  (mapcar #'buffer-name (beframe--get-buffers 'public)))
 
 (define-obsolete-function-alias
   'beframe--buffer-names
@@ -250,13 +303,14 @@ frame name."
     (mapcar
      (lambda (frame)
        (let ((name (car frame))
-             (obj (cdr frame)))
+             (obj (cdr frame))
+             (selected-frame (selected-frame)))
          (cond
-          ((and (eq (selected-frame) obj)
+          ((and (eq selected-frame obj)
                 (string-prefix-p " " name))
            (setcar frame "Current frame")
            frame)
-          ((eq (selected-frame) obj)
+          ((eq selected-frame obj)
            (setcar frame (format "%s (Current frame)" name))
            frame)
           (t
@@ -368,31 +422,50 @@ Also see the other Beframe commands:
       (beframe--frame-object (beframe--frame-prompt)))))
   (display-buffer (beframe-list-buffers-noselect frame :sort sort)))
 
-(defun beframe--assume (buffers)
-  "Assume BUFFERS into current frame.
-BUFFERS is a list of buffer objects.  If BUFFERS satisfies
-`framep', then the list of buffers is that of the corresponding
-frame object (per `beframe-buffer-list')."
-  (let ((buffer-list
-         (delete-dups
-          (append
-           (if (framep buffers)
-               (beframe-buffer-list buffers)
-             buffers)
-           (beframe-buffer-list)))))
-    (modify-frame-parameters nil `((buffer-list . ,buffer-list)))))
+(defun beframe--get-longest-list-first (list-1 list-2)
+  "Return a cons cell of LIST-1 and LIST-2 with the longest first."
+  (let ((length-1 (length list-1))
+        (length-2 (length list-2)))
+    (if (> length-1 length-2)
+        (cons list-1 list-2)
+      (cons list-2 list-1))))
 
-(defun beframe--unassume (buffers)
-  "Unassume BUFFERS from current frame.
-BUFFERS is a list of buffer objects.  If BUFFERS satisfies
-`framep', then the list of buffers is that of the corresponding
-frame object (per `beframe-buffer-list')."
-  (let ((buffer-list
-         (seq-filter
-          (lambda (buf)
-            (not (member buf (if (framep buffers) (beframe-buffer-list buffers) buffers))))
-          (beframe-buffer-list))))
-    (modify-frame-parameters nil `((buffer-list . ,buffer-list)))))
+(defun beframe--modify-buffer-list (operation buffers)
+  "Perform OPERATION to modify the current frame buffer list.
+
+OPERATION is a keyword to :assume or :unassume.  To assume is to include
+buffers into the buffer list.  To unassume is to remove them from the
+buffer list.
+
+BUFFERS is a list of buffer objects to be added or removed from the
+current frame buffer list.  If BUFFERS satisfies `framep', then the list
+of buffers is that of the corresponding frame object (per
+`beframe--get-buffers')."
+  (pcase-let* ((frame-buffers (beframe--get-buffers))
+               (new-buffers (if (framep buffers)
+                                (beframe--get-buffers buffers)
+                              buffers))
+               (`(,consolidated-buffers . ,action)
+                (pcase operation
+                  (:assume (cons (append new-buffers frame-buffers) "Assumed"))
+                  (:unassume (cons
+                              (seq-filter
+                               (lambda (buf)
+                                 (not (member buf new-buffers)))
+                               frame-buffers)
+                              "Unassumed"))
+                  (_ (error "`%s' is an unknown operation to modify frame buffers" operation)))))
+    (if-let ((lists (beframe--get-longest-list-first frame-buffers consolidated-buffers))
+             (difference (seq-difference
+                          (mapcar #'buffer-name (car lists))
+                          (mapcar #'buffer-name (cdr lists)))))
+        (progn
+          (modify-frame-parameters nil `((buffer-list . ,consolidated-buffers)))
+          (message "%s %s buffers: %s"
+                   (propertize action 'face 'error)
+                   (propertize (format "%s" (length difference)) 'face 'warning)
+                   (propertize (format "%s" difference) 'face 'success)))
+      (message "No change to the frame's buffer list"))))
 
 ;;;###autoload
 (defun beframe-assume-frame-buffers (frame)
@@ -404,7 +477,7 @@ Also see the other Beframe commands:
 
 \\{beframe-prefix-map}"
   (interactive (list (beframe--frame-object (beframe--frame-prompt))))
-  (beframe--assume frame))
+  (beframe--modify-buffer-list :assume frame))
 
 (make-obsolete
  'beframe-add-frame-buffers
@@ -421,7 +494,7 @@ Also see the other Beframe commands:
 
 \\{beframe-prefix-map}"
   (interactive (list (beframe--frame-object (beframe--frame-prompt))))
-  (beframe--unassume frame))
+  (beframe--modify-buffer-list :unassume frame))
 
 (make-obsolete
  'beframe-remove-frame-buffers
@@ -472,7 +545,7 @@ Also see the other Beframe commands:
      (beframe--buffer-list-prompt-crm
       (beframe--frame-object
        (beframe--frame-prompt))))))
-  (beframe--assume buffers))
+  (beframe--modify-buffer-list :assume buffers))
 
 (make-obsolete
  'beframe-add-buffers
@@ -489,11 +562,69 @@ Also see the other Beframe commands:
   "Like `beframe-assume-frame-buffers-selectively' but for all frames."
   (declare (interactive-only t))
   (interactive)
-  (beframe--assume
+  (beframe--modify-buffer-list
+   :assume
    (beframe--buffers-name-to-objects
     (beframe--buffer-list-prompt-crm
      :all-frames))))
 
+(defvar beframe-buffers-matching-regexp-history nil
+  "Minibuffer history of `beframe-buffers-matching-regexp-prompt'.")
+
+(defun beframe-buffers-matching-regexp-prompt (prompt-text)
+  "Prompt for regular with PROMPT-TEXT."
+  (let ((default (car beframe-buffers-matching-regexp-history)))
+    (read-regexp
+     (format-prompt prompt-text default)
+     default 'beframe-buffers-matching-regexp-history)))
+
+;;;###autoload
+(defun beframe-assume-buffers-matching-regexp (regexp &optional match-mode-names)
+  "Assume all buffers whose name matches REGEXP.
+With optional MATCH-MODE-NAMES return buffers whose name or major mode
+matches REGEXP.
+
+Also see the other Beframe commands:
+
+\\{beframe-prefix-map}"
+  (interactive
+   (let ((arg current-prefix-arg))
+     (list
+      (beframe-buffers-matching-regexp-prompt
+       (if arg
+           "Buffer names matching REGEXP in the name or major mode"
+         "Buffer names matching REGEXP in the name")))))
+  (if-let ((buffers (beframe--get-buffers (list regexp match-mode-names :no-internal-buffers))))
+      (beframe--modify-buffer-list :assume buffers)
+    (user-error "No buffers match `%s'" regexp)))
+
+(defalias 'beframe-assume-buffers-matching-regexp 'beframe-assume-buffers-matching-regexp-all-frames
+  "Alias for `beframe-assume-buffers-matching-regexp'.")
+
+;;;###autoload
+(defun beframe-unassume-buffers-matching-regexp (regexp &optional match-mode-names)
+  "Unassume all buffers whose name matches REGEXP.
+With optional MATCH-MODE-NAMES return buffers whose name or major mode
+matches REGEXP.
+
+Also see the other Beframe commands:
+
+\\{beframe-prefix-map}"
+  (interactive
+   (let ((arg current-prefix-arg))
+     (list
+      (beframe-buffers-matching-regexp-prompt
+       (if arg
+           "Buffer names matching REGEXP in the name or major mode"
+         "Buffer names matching REGEXP in the name")))))
+  (if-let ((buffers (beframe--get-buffers (list regexp match-mode-names :no-internal-buffers))))
+      (beframe--modify-buffer-list :unassume buffers)
+    (user-error "No buffers match `%s'" regexp)))
+
+
+(defalias 'beframe-unassume-buffers-matching-regexp 'beframe-unassume-buffers-matching-regexp-all-frames
+  "Alias for `beframe-unassume-buffers-matching-regexp'.")
+
 (define-obsolete-function-alias
   'beframe-unassume-buffers
   'beframe-unassume-current-frame-buffers-selectively
@@ -514,7 +645,7 @@ Also see the other Beframe commands:
    (list
     (beframe--buffers-name-to-objects
      (beframe--buffer-list-prompt-crm))))
-  (beframe--unassume buffers))
+  (beframe--modify-buffer-list :unassume buffers))
 
 (make-obsolete
  'beframe-remove-buffers
@@ -526,7 +657,7 @@ Also see the other Beframe commands:
   "Assume the consolidated buffer list (all frames)."
   (declare (interactive-only t))
   (interactive)
-  (beframe--assume (beframe--public-buffers)))
+  (beframe--modify-buffer-list :assume (beframe--get-buffers 'public)))
 
 ;;;###autoload
 (defun beframe-unassume-all-buffers-no-prompts ()
@@ -538,8 +669,35 @@ Also see the other Beframe commands:
 \\{beframe-prefix-map}"
   (declare (interactive-only t))
   (interactive)
-  (beframe--unassume (beframe--public-buffers))
-  (beframe--assume (beframe--global-buffers)))
+  (beframe--modify-buffer-list :unassume (beframe--get-buffers 'public))
+  (beframe--modify-buffer-list :assume (beframe--global-buffers)))
+
+;;;###autoload
+(defun beframe-kill-buffers-matching-regexp (regexp &optional match-mode-names)
+  "Delete all buffers whose name matches REGEXP.
+With optional MATCH-MODE-NAMES delete buffers whose name or major mode
+matches REGEXP.
+
+Note that this operation applies to all frames, because buffers are
+shared by the Emacs session even though Beframe only exposes those that
+pertain to a given frame.
+
+Also see the other Beframe commands:
+
+\\{beframe-prefix-map}"
+  (interactive
+   (let ((arg current-prefix-arg))
+     (list
+      (beframe-buffers-matching-regexp-prompt
+       (if arg
+           "Delete buffers matching REGEXP in the name or major mode"
+         "Delete buffers matching REGEXP in the name"))
+      arg)))
+  (if-let ((buffers (beframe--get-buffers (list regexp match-mode-names :no-internal-buffers))))
+      (when (or beframe-kill-buffers-no-confirm
+                (y-or-n-p (format "Kill %d buffers matching `%s'?" (length buffers) regexp)))
+        (mapc #'kill-buffer buffers))
+    (user-error "No buffers match `%s'" regexp)))
 
 ;;; Minor mode setup
 
@@ -550,13 +708,14 @@ Also see the other Beframe commands:
   "Keymap with Beframe commands.
 Meant to be assigned to a prefix key, like this:
 
-    (define-key global-map (kbd \"C-c b\") \=#'beframe-prefix-map)")
+    (define-key global-map (kbd \"C-c b\") \='beframe-prefix-map)")
 
 ;;;###autoload (autoload 'beframe-prefix-map "beframe")
 (define-prefix-command 'beframe-prefix-map)
 
 (define-key beframe-prefix-map (kbd "b") #'beframe-switch-buffer)
 (define-key beframe-prefix-map (kbd "m") #'beframe-buffer-menu)
+(define-key beframe-prefix-map (kbd "k") #'beframe-kill-buffers-matching-regexp)
 (define-key beframe-prefix-map (kbd "a f") #'beframe-assume-frame-buffers-selectively)
 (define-key beframe-prefix-map (kbd "a F") #'beframe-assume-frame-buffers)
 (define-key beframe-prefix-map (kbd "a a") #'beframe-assume-buffers-selectively-all-frames)
@@ -564,6 +723,8 @@ Meant to be assigned to a prefix key, like this:
 (define-key beframe-prefix-map (kbd "u f") #'beframe-unassume-current-frame-buffers-selectively)
 (define-key beframe-prefix-map (kbd "a F") #'beframe-unassume-frame-buffers)
 (define-key beframe-prefix-map (kbd "u U") #'beframe-unassume-all-buffers-no-prompts)
+(define-key beframe-prefix-map (kbd "a s") #'beframe-assume-buffers-matching-regexp)
+(define-key beframe-prefix-map (kbd "u s") #'beframe-unassume-buffers-matching-regexp)
 
 ;;;###autoload
 (define-minor-mode beframe-mode
@@ -619,25 +780,27 @@ Also see the variable `beframe-prefix-map'."
 (defun beframe-infer-frame-name (frame name)
   "Infer a suitable name for FRAME with given NAME.
 See `beframe-rename-frame'."
-  (when-let (((frame-list))
-             (buffer (car (frame-parameter frame 'buffer-list)))
-             (file-name (when (bufferp buffer) (buffer-file-name buffer)))
-             (buf-name (buffer-name buffer))
-             (dir (with-current-buffer buffer (or (vc-root-dir) default-directory)))
-             (projectp (and (bound-and-true-p project--list)
-                            (listp project--list)
-                            (member (list dir) project--list))))
-    (cond
-     ((and name (stringp name))
-      name)
-     ((and projectp buf-name)
-      (format "%s" (file-name-nondirectory (directory-file-name dir))))
-     ((and (not (minibufferp)) file-name)
-      (format "%s %s" buf-name dir))
-     ((not (minibufferp))
-      buf-name)
-     (t
-      dir))))
+  (when (frame-list)
+    (let* ((buffer (car (frame-parameter frame 'buffer-list)))
+           (file-name (when (bufferp buffer) (buffer-file-name buffer)))
+           (buf-name (buffer-name buffer))
+           (dir (with-current-buffer buffer (or (vc-root-dir) default-directory)))
+           (projectp (and (bound-and-true-p project--list)
+                          (listp project--list)
+                          (or
+                           (member (list dir) project--list)
+                           (member (list (abbreviate-file-name dir)) project--list)))))
+      (cond
+       ((and name (stringp name))
+        name)
+       ((and projectp buf-name)
+        (format "%s" (file-name-nondirectory (directory-file-name dir))))
+       ((and (not (minibufferp)) file-name)
+        (format "%s %s" buf-name dir))
+       ((not (minibufferp))
+        buf-name)
+       (t
+        dir)))))
 
 ;;;###autoload
 (defun beframe-rename-frame (frame &optional name)
@@ -714,7 +877,7 @@ If FRAME is nil, use the current frame."
 
 (defun beframe-do-not-assume-last-selected-buffer (&rest _)
   "Unassume the buffer of the most recently used window from the new frame."
-  (beframe--unassume (list (window-buffer (get-mru-window)))))
+  (beframe--modify-buffer-list :unassume (list (window-buffer (get-mru-window)))))
 
 (defun beframe--with-other-frame (&rest app)
   "Apply APP with `other-frame-prefix'.
